@@ -10,6 +10,8 @@ import numpy as np
 import os
 import sys
 import textwrap
+import matplotlib.pyplot as plt
+import matplotlib.ticker as ticker
 
 import hpc_suite as hpc
 from hpc_suite.action import OrderAction
@@ -18,9 +20,9 @@ from hpc_suite.generate_job import parse_hostname
 from . import gen_input
 from . import extractor
 from . import opt_extractor
-from . import freq_extractor
 from . import gen_job
 from . import cd_extractor
+from . import infrared
 
 
 def check_file(file_name, extension):
@@ -322,14 +324,17 @@ def distort_func(user_args):
     # Check log file has been provided
     check_file(user_args.log_file_name, ".log")
 
-    # Read displacement vectors from Gaussian
-    props, _ = freq_extractor.get_freq_prop_log(user_args.log_file_name)
+    dv_extractor = extractor.make_extractor(
+        user_args.log_file_name,
+        ("freq", "displacement")
+    )
+    disp_vecs = next((x for x in dv_extractor), None)[1]
 
     # Read optimised structure from Gaussian
     labels, coords = opt_extractor.read_coords_log(user_args.log_file_name)
 
     # Check user hasn't requested a wrong number
-    n_modes = np.shape(props["displacements"])[1]
+    n_modes = np.shape(disp_vecs)[1]
     if not 0 < user_args.mode <= n_modes:
         print('Invalid mode number selected')
         print('This molecule has {:d} modes'.format(n_modes))
@@ -337,9 +342,7 @@ def distort_func(user_args):
 
     # Distort along chosen vibrational mode by 1 unit of
     # normalised displacement vector
-    n_atoms = np.shape(props["displacements"])[0]//3
-    disp_vec = props["displacements"][:, user_args.mode-1]
-    disp_vec = np.reshape(disp_vec, [n_atoms, 3], order='C')
+    disp_vec = disp_vecs[user_args.mode-1]
     dist_coords = [co + di for co, di in zip(coords, disp_vec)]
 
     # Write .xyz file
@@ -366,49 +369,46 @@ def dip_deriv_func(user_args):
     None
 
     """
-    # Check log file has been provided
-    check_file(user_args.log_file_name, ".log")
-
-    # Check .fchk file has been provided
-    check_file(user_args.fchk_file_name, ".fchk")
-
-    # Extract displacement vectors for vibrational modes
-    props, _ = freq_extractor.get_freq_prop_log(
-        user_args.log_file_name
+    # Read reduced mass from log file
+    rm_extractor = extractor.make_extractor(
+        user_args.log_file_name,
+        ("freq", "reduced_mass")
     )
 
-    red_mass = props["reduced_masses"]
-    disp_vec = props["displacements"]
-    n_dirs = np.shape(disp_vec)[0]
+    red_masses = next((x for x in rm_extractor), None)[1]
+
+    # Read displacement vectors from fchk
+    dv_extractor = extractor.make_extractor(
+        user_args.fchk_file_name,
+        ("fchk", "displacement")
+    )
+
+    disp_vecs = next((x for x in dv_extractor), None)[1]
 
     # Remove sqrt(reduced_mass) weighting from each displacement vector
-    disp_vec /= np.tile(np.sqrt(red_mass), [n_dirs, 1])
+    # Dimensions are [3NAtoms, 3NAtoms-6]
+    disp_vecs /= np.sqrt(red_masses)
 
-    # Extract per atom dipole derivatives from .fchk file
-    dip_deriv_pa = freq_extractor.get_dip_deriv_fchk(
-        user_args.fchk_file_name
+    # Read dipole derivatives from fchk file
+    dd_extractor = extractor.make_extractor(
+        user_args.fchk_file_name,
+        ("fchk", "dipole_derivatives")
     )
-    n_modes = len(red_mass)
 
-    dip_deriv = np.zeros([n_modes, 3])
+    # Dimensions are
+    # [3NAtoms, 3]
+    dip_deriv_pa = next((x for x in dd_extractor), None)[1]
 
-    for it in range(n_modes):
-        dip_deriv[it] = np.sum(
-            dip_deriv_pa * np.tile(
-                disp_vec[:, it], [3, 1]
-                ).T,
-            axis=0
-        )
-
-    # Convert from e u^-1/2 to km^1/2 mol^-1/2
-    dip_deriv *= 31.2231
+    dip_deriv_modes = np.einsum('ji, jk->ki', dip_deriv_pa, disp_vecs)
+    # Convert to km^1/2 mol-1/2
+    dip_deriv_modes *= 31.2231
 
     out_head = user_args.log_file_name.split(".")[0]
     out_name = "{}_dipole_derivatives.dat".format(out_head)
 
     np.savetxt(
         out_name,
-        dip_deriv,
+        dip_deriv_modes,
         header="Units: km^1/2 mol^-1/2",
         fmt="% 7.5e"
     )
@@ -419,6 +419,86 @@ def dip_deriv_func(user_args):
     return
 
 
+def plot_infrared_func(user_args):
+    """
+    Wrapper function for CLI call to plot_infrared
+
+    Parameters
+    ----------
+    user_args : argparser object
+        User arguments
+
+    Returns
+    -------
+    None
+    """
+
+    f_extractor = extractor.make_extractor(
+        user_args.log_file_name,
+        ("freq", "frequency")
+    )
+
+    i_extractor = extractor.make_extractor(
+        user_args.log_file_name,
+        ("freq", "ir_intensity")
+    )
+
+    freqs = next((x for x in f_extractor), None)[1]
+    ints = next((x for x in i_extractor), None)[1]
+
+    f_range = np.linspace(
+        user_args.min_energy,
+        user_args.max_energy,
+        user_args.n_points
+    )
+
+    ls_funcs = {
+        "lorentzian": infrared.gen_lorentzian_spectrum,
+        "gaussian": infrared.gen_gaussian_spectrum
+    }
+
+    spectrum = ls_funcs[user_args.lineshape](
+        freqs, user_args.fwhm, ints, f_range
+    )
+
+    if user_args.save_data:
+        full = np.vstack([spectrum, f_range]).T
+        file_name = user_args.log_file_name.split(".log")[0]
+        file_name = "{}_ir_spectrum.dat".format(file_name)
+        np.savetxt(
+            file_name,
+            full,
+            header="Epsilon (L mol-1 cm-1), Energy (cm-1)"
+        )
+        print("Data saved to {}".format(file_name))
+
+    # Plot data
+    _, ax1 = plt.subplots(1, figsize=[6, 6])
+
+    ax1.plot(f_range, spectrum, lw=2)
+
+    ax1.spines.right.set_visible(False)
+    ax1.spines.top.set_visible(False)
+
+    ax1.set_xlabel("Energy (cm$^{-1}$)")
+    ax1.set_ylabel(r"$\epsilon$ (L mol$^{-1}$ cm$^{-1}$)")
+
+    ax1.xaxis.set_minor_locator(ticker.AutoMinorLocator())
+    ax1.yaxis.set_minor_locator(ticker.AutoMinorLocator())
+
+    ax1.set_xlim([
+        user_args.min_energy,
+        user_args.max_energy
+    ])
+
+    if user_args.save_plot:
+        plt.savefig("spectrum.png", dpi=400)
+    if not user_args.hide_plot:
+        plt.show()
+
+    return
+
+
 def cd_extractor_func(user_args):
     """
 
@@ -735,7 +815,7 @@ def read_args(arg_list=None):
         '--fchk',
         nargs='+',
         action=OrderAction,
-        choices=["hessian", "atomic_mass"],
+        choices=["hessian", "atomic_mass", "displacement"],
         help='Extract items from the formatted checkpoint file'
     )
 
@@ -847,6 +927,80 @@ def read_args(arg_list=None):
         help="Type of decomposition to extract from .log file"
     )
 
+    # Infrared spectrum
+
+    plot_ir = subparsers.add_parser(
+        "plot_ir",
+        formatter_class=RawTextHelpFormatter,
+        description="""
+        Plot infrared spectrum from log file
+        """
+    )
+
+    plot_ir.set_defaults(
+        func=plot_infrared_func
+    )
+
+    plot_ir.add_argument(
+        "log_file_name",
+        type=str,
+        help="Gaussian .log file name"
+    )
+
+    plot_ir.add_argument(
+        "--fwhm",
+        type=float,
+        default=15.,
+        help="Full Width Half-Maximum used for all peaks"
+    )
+
+    plot_ir.add_argument(
+        "--lineshape",
+        type=str,
+        choices=["lorentzian", "gaussian"],
+        default="gaussian",
+        help="Lineshape function used for spectrum"
+    )
+
+    plot_ir.add_argument(
+        "--min_energy",
+        type=float,
+        default=0.,
+        help="Minimum energy (cm-1) used to simulate spectrum"
+    )
+
+    plot_ir.add_argument(
+        "--max_energy",
+        type=float,
+        default=4000.,
+        help="Maximum energy (cm-1) used to simulate spectrum"
+    )
+
+    plot_ir.add_argument(
+        "--n_points",
+        type=int,
+        default=50000,
+        help="Number of points used to simulate spectrum"
+    )
+
+    plot_ir.add_argument(
+        "--save_plot",
+        action="store_true",
+        help="Save plot to file"
+    )
+
+    plot_ir.add_argument(
+        "--hide_plot",
+        action="store_true",
+        help="Stops plot from appearing on screen"
+    )
+
+    plot_ir.add_argument(
+        "--save_data",
+        action="store_true",
+        help="Saves data of raw spectrum to file"
+    )
+
     # Charges and dipoles extraction
 
     geom_conv = subparsers.add_parser(
