@@ -64,7 +64,9 @@ def make_extractor(f_gaussian, select):
     fchk_extractors = {
         ("fchk", "hessian"): "Cartesian Force Constants",
         ("fchk", "atomic_mass"): "Real atomic weights",
-        ("fchk", "coordinates"): "Current cartesian coordinates"
+        ("fchk", "coordinates"): "Current cartesian coordinates",
+        ("fchk", "dipole_derivatives"): "Dipole Derivatives",
+        ("fchk", "displacement"): "Vib-Modes"
     }
 
     if is_fchk(f_gaussian):
@@ -136,9 +138,9 @@ class ExtractFreq(PlainTextExtractor):
 
         chunk_pattern = (
             ''.join(('(?P<data>' if label == item else '') +
-                    ('({}\ +\-\-\-\ +{}(\ +{})+\s+)?'.format(*pat, pat[1])
+                    ('({}\ +\-\-\-\ +{}(\ +{})+\s+)?'.format(*pat, pat[1]) # noqa
                     if pat[0] else
-                    '({}(\ +{})+\s+)?'.format(pat[1], pat[1])) +
+                    '({}(\ +{})+\s+)?'.format(pat[1], pat[1])) +  # noqa
                     (')' if label == item else '')
                     for item, pat in self.pattern.items()
                     if item != "displacement") +
@@ -219,17 +221,23 @@ class FchkExtractor(PlainTextExtractor):
         description = {
             "Cartesian Force Constants": "Cartesian Hessian matrix",
             "Real atomic weights": "atomic masses",
-            "Current cartesian corrdinates": "Cartesian coordinates"
+            "Current cartesian corrdinates": "Cartesian coordinates",
+            "Dipole Derivatives": "Dipole Derivatives",
+            "Vib-Modes": "Vibrational Mode Vectors",
         }
 
         units = {
-            "Real atomic weights": 'amu'
+            "Real atomic weights": 'amu',
+            "Dipole Derivatives": "e Angstrom^-1",
+            "Vib-Modes": "amu^-(1/2)",
         }
 
         fmt = {
             "Cartesian Force Constants": '% 16.8e',
             "Real atomic weights": '% 16.8e',
-            "Current cartesian corrdinates": '% 16.8e'
+            "Current cartesian corrdinates": '% 16.8e',
+            "Dipole Derivatives": '% 16.8e',
+            "Vib-Modes": '% 16.8e',
         }
 
         self.item = item
@@ -279,11 +287,21 @@ class FchkExtractor(PlainTextExtractor):
             out[mask] = arr
             return out + np.tril(out, k=-1).T
 
+        def vib_quadratic_solve(n_ele):
+            N = (18 + np.sqrt(324 + (36*n_ele)))/(18)
+            return (3*int(N), 3*int(N)-6)
+
         func = {
             'Cartesian Force Constants': fill_lower_triangle,
             'Real atomic weights': lambda x: np.array(x, dtype=float),
             'Current cartesian coordinates':
-                lambda x: np.reshape(x, (-1, 3)).astype(float)
+                lambda x: np.reshape(x, (-1, 3)).astype(float),
+            'Dipole Derivatives':  # (3N * 3)
+                lambda x: np.reshape(x, (np.size(x)//3, 3)).astype(float),
+            'Vib-Modes':  # (3N * 3N-6)
+                lambda x: np.reshape(
+                    x, vib_quadratic_solve(np.size(x)), order='F'
+                ).astype(float)
         }
 
         return func[self.item](data)
