@@ -10,7 +10,6 @@ from matplotlib.ticker import MaxNLocator
 import xyz_py as xyzp
 import xyz_py.atomic
 import sys
-from . import freq_extractor
 
 
 def read_coords_fchk(f_name, numbered=False):
@@ -36,7 +35,7 @@ def read_coords_fchk(f_name, numbered=False):
     a_nums = []
     coords = []
 
-    n_atoms = freq_extractor.get_n_atoms_fchk(f_name)
+    n_atoms = get_n_atoms_fchk(f_name)
 
     # Open .fchk file
     with open(f_name, "r") as f:
@@ -90,7 +89,7 @@ def read_coords_log(f_name, numbered=False):
             (n_atom,3) array of atomic coordinates
     """
 
-    n_atoms = freq_extractor.get_n_atoms_log(f_name)
+    n_atoms = get_n_atoms_log(f_name)
 
     # Get coordinates
     with open(f_name, "r") as f:
@@ -357,3 +356,65 @@ def plot_max_rmsd_force_displacement(props, thresholds, show=True, save=False,
         plt.show()
 
     return fig, ax1, ax2, ax3, ax4
+
+
+def get_n_atoms_log(f_name):
+    """
+    Extracts number of atoms from Gaussian log file
+
+    Parameters
+    ----------
+        f_name : str
+            Name of log file
+
+    Returns
+    -------
+        int
+            Number of atoms
+    """
+    n_atoms = 0
+
+    with open(f_name, "r") as f:
+        for line in f:
+            if "NAtoms=" in line:
+                spl_line = line.split()
+                n_atoms = int(spl_line[spl_line.index("NAtoms=")+1])
+                break
+
+    if n_atoms == 0:
+        print("Cannot find number of atoms in file {}".format(f_name))
+        print("Aborting")
+        exit()
+
+    return n_atoms
+
+
+def get_n_atoms_fchk(f_name):
+    """
+    Extracts number of atoms from Gaussian fchk file
+
+    Parameters
+    ----------
+        f_name : str
+            Name of log file
+
+    Returns
+    -------
+        int
+            Number of atoms
+    """
+    n_atoms = 0
+
+    with open(f_name, "r") as f:
+        for line in f:
+            if "Number of atoms" in line:
+                spl_line = line.split()
+                n_atoms = int(spl_line[4])
+                break
+
+    if n_atoms == 0:
+        print("Cannot find number of atoms in file {}".format(f_name))
+        print("Aborting")
+        exit()
+
+    return n_atoms
